import os
from datetime import datetime

os.system("") #

class colors:

    RESET = "\033[0m"

    BOLD = "\033[1m"
    ITALIC = "\033[3m"
    UNDERLINE = "\033[4m"

    BLACK = "\033[30m\033[47m"
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    PURPLE = "\033[35m"
    AQUA = "\033[36m"
    WHITE = "\033[37m"
    #GRAY = "\033[90m"

    LIGHT_RED = "\033[91m"
    LIGHT_GREEN = "\033[92m"
    LIGHT_YELLOW = "\033[93m"
    LIGHT_BLUE = "\033[94m"
    LIGHT_PURPLE = "\033[95m"
    LIGHT_AQUA = "\033[96m"
    LIGHT_WHITE = "\033[97m"

    BLACK_BG = "\033[40m"
    RED_BG = "\033[41m"
    GREEN_BG = "\033[42m"
    YELLOW_BG = "\033[43m"
    BLUE_BG = "\033[44m"
    PURPLE_BG = "\033[45m"
    CYAN_BG = "\033[46m"
    WHITE_BG = "\033[47m"

def _values_to_string(*values: object, sep: str = ' '): return sep.join([str(v) for v in values])

class Logger:

    def __init__(self, logs_path: str = '.polinv-logs') -> None:
        self.logs_path = logs_path

    def save_today_log(self, text: str):
        path = os.path.join(self.logs_path, 'today.log')
        with open(path, 'a', encoding='utf-8') as file: file.write(text)

    def log(self, text: str, color: str = colors.RESET, save: bool = False):
        log_time = datetime.now().strftime("%H:%M:%S")
        if save: self.save_today_log(log_time + ' > ' + text + '\n')
        print(f'{color}{text}{colors.RESET}')

    def error(self, text: str):
        log_time = datetime.now().strftime("%H:%M:%S")
        self.save_today_log('(error) -> ' + log_time + ' > ' + text + '\n')
        print(f'{colors.RED}(error) -> {colors.LIGHT_RED}' + text + f'{colors.RESET}')

    def info(self, *values: object, sep: str = ' ', save: bool = False): self.log(f'(INFO) {_values_to_string(*values, sep)}', colors.LIGHT_AQUA, save)
    def success(self, *values: object, sep: str = ' ', save: bool = False): self.log(f'(INFO) {_values_to_string(*values, sep)}', colors.LIGHT_GREEN, save)
    def primary(self, *values: object, sep: str = ' ', save: bool = False): self.log(f'(INFO) {_values_to_string(*values, sep)}', colors.LIGHT_PURPLE, save)
    def warning(self, *values: object, sep: str = ' ', save: bool = False): self.log(f'(WARNING) {_values_to_string(*values, sep)}', colors.LIGHT_YELLOW, save)
    def danger(self, *values: object, sep: str = ' ', save: bool = False): self.log(f'(WARNING) {_values_to_string(*values, sep)}', colors.LIGHT_RED, save)

    def info_profile(self, profile: str, *values: object, sep: str = ' ', save: bool = False): self.info(f'({profile}) -> {_values_to_string(*values, sep)}', sep=sep, save=save)
    def success_profile(self, profile: str, *values: object, sep: str = ' ', save: bool = False): self.success(f'({profile}) -> {_values_to_string(*values, sep)}', sep=sep, save=save)
    def primary_profile(self, profile: str, *values: object, sep: str = ' ', save: bool = False): self.primary(f'({profile}) -> {_values_to_string(*values, sep)}', sep=sep, save=save)
    def warning_profile(self, profile: str, *values: object, sep: str = ' ', save: bool = False): self.warning(f'({profile}) -> {_values_to_string(*values, sep)}', sep=sep, save=save)
    def danger_profile(self, profile: str, *values: object, sep: str = ' ', save: bool = False): self.danger(f'({profile}) -> {_values_to_string(*values, sep)}', sep=sep, save=save)

def save_today_log(text: str):
    path = os.path.join('.polinv-logs', 'today.log')
    with open(path, 'a', encoding='utf-8') as file: file.write(text)

def log(text: str, color: str = colors.RESET, save: bool = False):
    log_time = datetime.now().strftime("%H:%M:%S")
    if save: save_today_log(log_time + ' > ' + text + '\n')
    print(f'{color}{text}{colors.RESET}')

def error(text: str):
    log_time = datetime.now().strftime("%H:%M:%S")
    save_today_log('(error) -> ' + log_time + ' > ' + text + '\n')
    print(f'{colors.RED}(error) -> {colors.LIGHT_RED}' + text + f'{colors.RESET}')

def info(*values: object, sep: str = ' ', save: bool = False): log(f'(INFO) {_values_to_string(*values, sep)}', colors.LIGHT_AQUA, save)
def success(*values: object, sep: str = ' ', save: bool = False): log(f'(INFO) {_values_to_string(*values, sep)}', colors.LIGHT_GREEN, save)
def primary(*values: object, sep: str = ' ', save: bool = False): log(f'(INFO) {_values_to_string(*values, sep)}', colors.LIGHT_PURPLE, save)
def warning(*values: object, sep: str = ' ', save: bool = False): log(f'(WARNING) {_values_to_string(*values, sep)}', colors.LIGHT_YELLOW, save)
def danger(*values: object, sep: str = ' ', save: bool = False): log(f'(WARNING) {_values_to_string(*values, sep)}', colors.LIGHT_RED, save)

def info_profile(profile: str, *values: object, sep: str = ' ', save: bool = False): info(f'({profile}) -> {_values_to_string(*values, sep)}', sep=sep, save=save)
def success_profile(profile: str, *values: object, sep: str = ' ', save: bool = False): success(f'({profile}) -> {_values_to_string(*values, sep)}', sep=sep, save=save)
def primary_profile(profile: str, *values: object, sep: str = ' ', save: bool = False): primary(f'({profile}) -> {_values_to_string(*values, sep)}', sep=sep, save=save)
def warning_profile(profile: str, *values: object, sep: str = ' ', save: bool = False): warning(f'({profile}) -> {_values_to_string(*values, sep)}', sep=sep, save=save)
def danger_profile(profile: str, *values: object, sep: str = ' ', save: bool = False): danger(f'({profile}) -> {_values_to_string(*values, sep)}', sep=sep, save=save)



import os
import cloudscraper

from onego import files

class SessionChrome(cloudscraper.CloudScraper):

    def __init__(self, profile: str, chrome_version: int = 131, cookies_path: str = '.polinv-session-cookies', *args, **kwargs):
        self.profile = profile
        self.cookies_path = os.path.join(cookies_path, f'{self.profile}.json')
        self.chrome_version = chrome_version or 131

        os.makedirs(cookies_path, exist_ok=True)

        super().__init__(*args, **kwargs)

        user_agent = f'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{self.chrome_version}.0.0.0 Safari/537.36'

        self.headers.update({ 'User-Agent': user_agent })
        self.headers.update({ 'Sec-Ch-Ua': f'"Not_A Brand";v="8", "Chromium";v="{self.chrome_version}", "Google Chrome";v="{self.chrome_version}"' })
        self.headers.update({ 'Sec-Ch-Ua-Mobile': '?0' })
        self.headers.update({ 'Sec-Ch-Ua-Platform': '"Windows"' })

    def __info(self): return f'<Session profile={self.profile} />'

    def save_cookies(self):
        cookies = self.export_cookies_dict()
        files.writeJSON(self.cookies_path, cookies)

    def load_cookies(self):
        cookies = files.readJSON(self.cookies_path)
        if not cookies: return []
        for c in cookies: self.cookies.set(c['name'], c['value'], domain=c['domain'], path=c['path'], secure=c['secure'], expires=c['expires'])
        return cookies

    def export_cookies_dict(self):
        return [{ 'name': c.name, 'value': c.value, 'domain': c.domain, 'path': c.path, 'secure': c.secure, 'expires': c.expires } for c in self.cookies]

    def export_cookies(self):
        return [c for c in self.cookies]

    def __str__(self): return self.__info()
    def __repr__(self): return self.__info()

def make_session(profile: str, *args, **kwargs): return SessionChrome(profile, *args, **kwargs)



import os
import csv
import json
import pickle

def read(path: str, mode: str = 'r') -> str:
    if not os.path.isfile(path): return
    encoding = None if mode == 'rb' else 'utf-8'
    with open(path, mode, encoding=encoding) as file: return file.read()

def readJSON(path: str) -> dict:
    if not os.path.isfile(path): return
    with open(path, 'r', encoding='utf-8') as file: return json.load(file)

def readPickle(path: str):
    if not os.path.isfile(path): return
    with open(path, 'rb') as file: return pickle.load(file)

def readCSV(path: str):
    file = open(path, 'r', encoding='utf-8')
    reader = csv.reader(file)
    return reader, file

def write(path: str, data: str, mode: str = 'w') -> None:
    encoding = None if mode == 'wb' else 'utf-8'
    with open(path, mode, encoding=encoding) as file: return file.write(data)

def writeJSON(path: str, data: dict) -> None:
    with open(path, 'w', encoding='utf-8') as file: return json.dump(data, file)

def writePickle(path: str, data):
    with open(path, 'wb') as file: return pickle.dump(data, file)

def writeCSV(path: str, rows: list[list[str]]):
    with open(path, 'w', encoding='utf-8', newline='') as file:
        writer = csv.writer(file)
        for row in rows: writer.writerow(row)

def join(path: str, *paths: str): return os.path.join(path, *paths)



from setuptools import setup, find_packages

# with open('README.md', 'r', encoding='utf-8') as f:
#     readme = f.read()

VERSION = '1.1.1'

setup(
    name='onego',
    version=VERSION,
    description='Cool •-•',
    # long_description=readme,
    # long_description_content_type="text/markdown",
    license='Apache 2.0',
    author='th3poli',
    author_email='',
    keywords=['onego', 'all in one'],
    packages=find_packages(),
    # include_package_data=True,
    url='https://github.com/th3poli/onegopy',
    install_requires=[],
    extras_require={
        'sessions': ['cloudscraper', 'onego[files]'],
        'files': [],
        'logger': [],
        'utils': ['requests', 'beautifulsoup4'],
        # 'chrome_driver': ['undetected-chromedriver'],
        'all': [
            'requests',
            'cloudscraper',
            'beautifulsoup4',
            # 'undetected-chromedriver'
        ],
    },
)

# OLD: python setup.py sdist bdist_wheel
# OLD: pip install ./dist/onego-1.0.0.tar.gz --no-cache-dir

# NEW: python -m build
# NEW: pip install ./dist/onego-1.1.0-py3-none-any.whl --no-cache-dir
# python -m twine upload .\dist\*

# import os;print('Local install command -> pip3 install ' + os.path.join(os.getcwd(), 'dist', f'onego-{VERSION}.tar.gz --no-cache-dir'))
# import os;print('Local install command -> pip3 install ' + os.path.join(os.getcwd(), 'dist', f'onego-{VERSION}-py3-none-any.whl --no-cache-dir'))


import time
import random
import requests
import threading
from bs4 import BeautifulSoup

from onego import logger

def random_sleep(min: int = 1, max: int = 5): time.sleep(random.randint(min, max))

def create_thread_groups(list_: list, number_of_groups: int = 4):
    groups = []
    for _ in range(number_of_groups): groups.append([])
    n = 0
    for item in list_:
        groups[n].append(item)
        if n >= number_of_groups - 1: n = 0
        else: n += 1
    return groups

def make_threads(func, args: list[tuple]) -> list[threading.Thread]:
    threads = []
    for args_ in args: threads.append(threading.Thread(target=func, args=args_))
    return threads

def start_thread_groups(threads: list[threading.Thread]):
    for t in threads: t.start()

def join_thread_groups(threads: list[threading.Thread]):
    for t in threads: t.join()

def parse(res: requests.Response): return BeautifulSoup(res.text, 'html.parser')

def str_contains_any(text: str, items: list[str]):
    for e in items:
        if str(e) in text: return True
    return False

def analyze_form(form: BeautifulSoup):
    form_data = { 'powrot': '1' }

    # Zbieranie wszystkich pól input, textarea, select z formularza
    for input_field in form.find_all(['input', 'textarea', 'select']):
        name = input_field.get('name')
        
        # Sprawdź, czy pole ma atrybut 'name', jeśli nie, to je pomijamy
        if not name:
            continue
        
        # Dla inputów sprawdzamy typ
        if input_field.name == 'input':
            input_type = input_field.get('type', 'text')  # Domyślnie 'text', jeśli typ nie jest podany

            if input_type in ['text', 'hidden', 'password', 'email', 'number', 'radio', 'submit', 'button', 'image']:
                # Dla input typu radio wybieramy tylko zaznaczone pole
                if input_type == 'radio':
                    if input_field.has_attr('checked'):
                        form_data[name] = input_field.get('value', '')
                else:
                    # Dla innych typów input pobieramy wartość
                    form_data[name] = input_field.get('value', '')

            elif input_type == 'checkbox':
                # Dla checkboxów zbieramy wszystkie zaznaczone wartości
                if input_field.has_attr('checked'):
                    if name in form_data:
                        if isinstance(form_data[name], list):
                            form_data[name].append(input_field.get('value', 'on'))
                        else:
                            form_data[name] = [form_data[name], input_field.get('value', 'on')]
                    else:
                        form_data[name] = input_field.get('value', 'on')

            elif input_type == 'file':
                # Dla plików wartość będzie zazwyczaj pusta, ponieważ pliki są przesyłane osobno
                form_data[name] = input_field.get('value', '')

        # Obsługa textarea
        elif input_field.name == 'textarea':
            form_data[name] = input_field.text

        # Obsługa select
        elif input_field.name == 'select':
            if input_field.has_attr('multiple'):
                # Zbieramy wszystkie zaznaczone opcje w select z multiple
                selected_options = [[option.get('value', ''), option.text] for option in input_field.find_all('option')] # selected=True
                form_data[name] = selected_options
            else:
                # Zbieramy tylko jedną wybraną opcję w select bez multiple
                selected_option = input_field.find('option', selected=True)
                if selected_option:
                    form_data[name] = selected_option.get('value', '')
                else:
                    first_option = input_field.find('option')
                    if first_option:
                        form_data[name] = first_option.get('value', '')

    return form_data

def calculate_time(func):
    def wrapper(*args, **kwargs):
        s = time.time()
        res = func(*args, **kwargs)
        e = time.time()
        logger.info(f'Execution time for function {func.__name__}: {round(e - s, 2)}s')
        return res
    return wrapper

class WaitFlag:

    def __init__(self, initial_value: bool = True) -> None:
        self.flag = initial_value
        self.initial_value = initial_value

    def reset(self): self.flag = self.initial_value

    def wait(self, interval_in_seconds: int = 1):
        while self.flag: time.sleep(interval_in_seconds)
    
    def stop(self): self.flag = not self.initial_value


#
# Programmatically detect the version of the Chrome web browser installed on the PC.
# Compatible with Windows, Mac, Linux.
# Written in Python.
# Uses native OS detection. Does not require Selenium nor the Chrome web driver.
#

import os
import re
from sys import platform

def extract_version_registry(output):
    try:
        google_version = ''
        for letter in output[output.rindex('DisplayVersion    REG_SZ') + 24:]:
            if letter != '\n':
                google_version += letter
            else:
                break
        return(google_version.strip())
    except TypeError:
        return

def extract_version_folder():
    # Check if the Chrome folder exists in the x32 or x64 Program Files folders.
    for i in range(2):
        path = 'C:\\Program Files' + (' (x86)' if i else '') +'\\Google\\Chrome\\Application'
        if os.path.isdir(path):
            paths = [f.path for f in os.scandir(path) if f.is_dir()]
            for path in paths:
                filename = os.path.basename(path)
                pattern = r'\d+\.\d+\.\d+\.\d+'
                match = re.search(pattern, filename)
                if match and match.group():
                    # Found a Chrome version.
                    return match.group(0)

    return None

def get_chrome_version():
    version = None
    install_path = None

    try:
        if platform == "linux" or platform == "linux2":
            # linux
            install_path = r"/usr/bin/google-chrome"
        elif platform == "darwin":
            # OS X
            install_path = r"/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome"
        elif platform == "win32":
            # Windows...
            try:
                # Try registry key.
                stream = os.popen('reg query "HKLM\\SOFTWARE\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Google Chrome"')
                output = stream.read()
                version = extract_version_registry(output)
            except Exception as ex:
                # Try folder path.
                version = extract_version_folder()

    except Exception as ex: pass

    version = os.popen(f"{install_path} --version").read().strip('Google Chrome ').strip() if install_path else version

    return version

def get_chrome_version_soft():
    chrome_version = get_chrome_version()
    return int(chrome_version.split('.')[0]) if chrome_version else None
